package com.hongyegroup.cpt_main.ui;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.Instrumentation;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.JavascriptInterface;
import android.webkit.ValueCallback;
import android.webkit.WebBackForwardList;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.guadou.cs_cptservices.YYConstants;
import com.guadou.cs_cptservices.widget.PickPhotoDialog;
import com.guadou.lib_baselib.base.activity.BaseVMActivity;
import com.guadou.lib_baselib.base.vm.BaseViewModel;
import com.guadou.lib_baselib.engine.image_select.ImageSelectExtKt;
import com.guadou.lib_baselib.engine.permission.PermissionExtKt;
import com.guadou.lib_baselib.engine.permission.UriExtKt;
import com.guadou.lib_baselib.utils.CheckUtil;
import com.guadou.lib_baselib.utils.CommUtils;
import com.guadou.lib_baselib.utils.SPUtils;
import com.guadou.lib_baselib.utils.ThreadPoolUtils;
import com.guadou.lib_baselib.utils.log.YYLogUtils;
import com.guadou.lib_baselib.view.MyWebView;
import com.guadou.lib_baselib.view.WebViewManager;
import com.guadou.lib_baselib.view.WebViewProgressBar;
import com.hongyegroup.cpt_main.R;
import com.luck.picture.lib.entity.LocalMedia;
import com.luck.picture.lib.listener.OnResultCallbackListener;
import com.luck.picture.lib.tools.ToastUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import kotlin.Unit;
import kotlin.jvm.functions.Function0;

/**
 * V11 Me页面的帮助中心
 * 内部处理
 */
public class HelpCenterProfileH5Activity extends BaseVMActivity<BaseViewModel> {

    private String mWebtitle;
    private String mWeburl;
    private WebViewProgressBar mPbWebView;
    private FrameLayout mFlContent;
    private MyWebView mWebView;
    private TextView mTvTitle;
    private ImageView mIvBack;
    private ImageView mIvFinish;
    private ImageView mIvSearch;

    private String mHelpCenterUrl = "";

    public static void startInstance(String helpCenterUrl) {
        Context context = CommUtils.getContext();
        Intent intent = new Intent(context, HelpCenterProfileH5Activity.class);
        intent.putExtra("helpCenterUrl", helpCenterUrl);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(intent);
    }

    @Override
    protected void getDataFromIntent(Intent intent) {
        super.getDataFromIntent(intent);
        mHelpCenterUrl = intent.getStringExtra("helpCenterUrl");
    }

    @Override
    public void init(@Nullable Bundle savedInstanceState) {

        mWebtitle = "Help Center";

        mWeburl = mHelpCenterUrl + "?token=" + SPUtils.getInstance(CommUtils.getContext()).getString(YYConstants.USER_TOKEN, "");

        initView();
        initTitles();
        initWeb();
        initListener();

    }

    @Override
    public int getLayoutIdRes() {

        return R.layout.activity_help_center;

    }

    @Override
    public void startObserve() {

    }

    private void initListener() {
        mIvBack.setOnClickListener(v -> {
            //走底部系统返回键
            webgoBack();
        });

        mIvFinish.setOnClickListener(v -> {
            finish();
        });

        mIvSearch.setOnClickListener(v -> {
            if (mWebView != null) {
                mWebView.loadUrl(mHelpCenterUrl + "/search");
            }
        });
    }

    private void webgoBack() {
        if (ThreadPoolUtils.getCachedThreadPool() != null) {
            ThreadPoolUtils.getCachedThreadPool().execute(() -> {
                try {
                    Instrumentation inst = new Instrumentation();
                    inst.sendKeyDownUpSync(KeyEvent.KEYCODE_BACK);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });
        }
    }

    private void initTitles() {
        if (!CheckUtil.isEmpty(mWebtitle) && mTvTitle != null) {
            mTvTitle.setText(mWebtitle);
        }
    }

    @SuppressLint("AddJavascriptInterface")
    private void initWeb() {
        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);

        mWebView = WebViewManager.obtain(CommUtils.getContext());
        mWebView.setLayoutParams(params);

        mWebView.addJavascriptInterface(new H5CallBackAndroid(), "webkit");

        //文件与图片的选择回调
        mWebView.setOnWebChooseFileListener(new MyWebView.OnWebChooseFileListener() {
            @Override
            public void onWebFileSelect(ValueCallback<Uri[]> callback) {
                filePathCallback1 = callback;
                showPickDialog();
            }
        });

        //网络状态回调
        mWebView.setOnWebChangeListener(new MyWebView.OnWebChangeListener() {
            @Override
            public void titleChange(String title) {

                if (!CheckUtil.isEmpty(title) && mTvTitle != null && !title.equals("common_html")) {
                    mTvTitle.setText(title);
                    if (title.equals("Help Center")) {
                        mIvSearch.setVisibility(View.VISIBLE);
                    } else {
                        mIvSearch.setVisibility(View.GONE);
                    }
                }
            }

            @Override
            public void progressChange(int newProgress) {

                if (newProgress == 100) {
                    mPbWebView.setProgress(100);
                    CommUtils.getHandler().postDelayed(() -> mPbWebView.setVisibility(View.GONE), 200);//0.2秒后隐藏进度条
                } else if (mPbWebView.getVisibility() == View.GONE) {
                    mPbWebView.setVisibility(View.VISIBLE);
                }
                //设置初始进度10，这样会显得效果真一点，总不能从1开始吧
                if (newProgress < 10) {
                    newProgress = 10;
                }
                //不断更新进度
                mPbWebView.setProgress(newProgress);
            }

            @Override
            public void onInnerLinkChecked() {
                mIvFinish.setVisibility(View.VISIBLE);
                YYLogUtils.e("onInnerLinkChecked" + mWebView.canGoBack());
            }

            @Override
            public void onWebLoadError() {
                ToastUtils.s(mActivity, "Load Error!");
            }
        });


        YYLogUtils.e("mWeburl:" + mWeburl);
        if (!TextUtils.isEmpty(mWeburl))
            mWebView.loadUrl(mWeburl);

        mFlContent.addView(mWebView);
    }

    private boolean isCancel = true;

    /**
     * 相机相册的选择
     */
    private void showPickDialog() {

        PickPhotoDialog photoDialog = new PickPhotoDialog(mActivity);
        photoDialog.SetOnChooseClickListener(new PickPhotoDialog.OnChooseClickListener() {
            @Override
            public void chooseCamera() {
                isCancel = false;
                startCamera();
            }

            @Override
            public void chooseAlbum() {
                isCancel = false;
                startAlbum();
            }
        });
        photoDialog.setCancelable(false);
        photoDialog.show();
        photoDialog.setOnDismissListener(dialog -> {
            cancelFilePick();
        });
    }

    //选择相册
    private void startAlbum() {

        PermissionExtKt.extRequestPermission(mActivity, new String[]{Manifest.permission.CAMERA, Manifest.permission.WRITE_EXTERNAL_STORAGE}, new Function0<Unit>() {
            @Override
            public Unit invoke() {

                ImageSelectExtKt.extOpenImageSelect(mActivity, new ArrayList<LocalMedia>(), new OnResultCallbackListener<LocalMedia>() {

                    @Override
                    public void onResult(List<LocalMedia> result) {
                        handlePath(result);
                    }

                    @Override
                    public void onCancel() {
                        cancelFilePick();
                    }
                }, 1, false, false, true, 1, 1, false);

                return null;
            }
        });
    }

    //选择相机
    private void startCamera() {

        PermissionExtKt.extRequestPermission(mActivity, new String[]{Manifest.permission.CAMERA, Manifest.permission.WRITE_EXTERNAL_STORAGE}, new Function0<Unit>() {
            @Override
            public Unit invoke() {

                ImageSelectExtKt.extOpenCamera(mActivity, new ArrayList<LocalMedia>(), new OnResultCallbackListener<LocalMedia>() {
                    @Override
                    public void onResult(List<LocalMedia> result) {
                        handlePath(result);
                    }

                    @Override
                    public void onCancel() {
                        cancelFilePick();
                    }
                }, false, true, 1, 1);

                return null;
            }
        });

    }

    /**
     * 处理图片-转换图片-返回给Web
     */
    private void handlePath(List<LocalMedia> result) {
        YYLogUtils.w("处理图片-转换图片-返回给Web");

        if (!CheckUtil.isEmpty(result)) {

            String path = result.get(0).getCompressPath();

//            Uri fileUri = Uri.fromFile(new File(path));
//            Uri fileUri = AndPermission.getFileUri(mActivity, new File(path));
            Uri fileUri = UriExtKt.getFileUri(this, new File(path));
            YYLogUtils.w("fileUri:" + fileUri + " path:" + path);

            if (filePathCallback1 != null) {
                filePathCallback1.onReceiveValue(new Uri[]{fileUri});
                filePathCallback1 = null;
            }
        }

        isCancel = true;
    }

    //取消图片的选择
    private void cancelFilePick() {
        if (isCancel && filePathCallback1 != null) {
            YYLogUtils.w("取消图片的选择");
            filePathCallback1.onReceiveValue(null);
            filePathCallback1 = null;
        }
        isCancel = true;
    }


    //JS Java Call 用一个类包装起来方便忽略混淆
    private class H5CallBackAndroid {

        @JavascriptInterface
        public void callPhone(String phoneNum) {

            PermissionExtKt.extRequestPermission(mActivity, new String[]{Manifest.permission.CALL_PHONE}, new Function0<Unit>() {
                @Override
                public Unit invoke() {

                    call();

                    return null;
                }
            });

        }

        private void call() {

            Intent intent = new Intent();
            intent.setAction(Intent.ACTION_CALL);
            Uri uri = Uri.parse("tel:" + "+65 66046919");
            intent.setData(uri);
            startActivity(intent);

        }

        @JavascriptInterface
        public void EmailUsSuccess(Object obj) {
            YYLogUtils.e("回调EmailUsSuccess");
            webgoBack();
        }
    }

    private void initView() {
        mPbWebView = findViewById(R.id.pb_web_view);
        mFlContent = findViewById(R.id.fl_content);
        mTvTitle = findViewById(R.id.tv_title);
        mIvBack = findViewById(R.id.iv_back);
        mIvFinish = findViewById(R.id.iv_finish);
        mIvSearch = findViewById(R.id.iv_search);
    }

    /**
     * 回调的图片选择
     */
    private ValueCallback<Uri[]> filePathCallback1;

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        //99为之前设置中的标识
        if (requestCode == 99) {
            if (resultCode == RESULT_OK && null != data) {
                Uri selectedImage = data.getData();
                filePathCallback1.onReceiveValue(new Uri[]{selectedImage});
            } else {
                //没有选择图片,返回空,不返回的话,点击过一次以后,无法再启动相册
                filePathCallback1.onReceiveValue(new Uri[]{});
            }
        }
    }

    /**
     * 集成了系统的回退和网页的回退
     */
    public boolean onKeyDown(int keyCode, KeyEvent event) {

        WebBackForwardList webBackForwardList = mWebView.copyBackForwardList();
        String historyOneOriginalUrl = webBackForwardList.getItemAtIndex(0).getOriginalUrl();
        int curIndex = webBackForwardList.getCurrentIndex();

        if ((keyCode == KeyEvent.KEYCODE_BACK) && mWebView.canGoBack()) {

            //判断是否是缓存的WebView
            if (historyOneOriginalUrl.contains("data:text/html;charset=utf-8")) {
                //说明是缓存复用的的WebView
                if (curIndex > 1) {
                    //内部跳转到另外的页面了，可以返回的
                    mWebView.goBack();
                    return true;
                } else {
                    //等于1的时候就要Finish页面了
                    return super.onKeyDown(keyCode, event);
                }

            } else {
                //如果不是缓存复用的WebView，可以直接返回
                mWebView.goBack();
                return true;
            }

        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (mWebView != null) {
            mWebView.onPause();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (mWebView != null) {
            mWebView.onResume();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        WebViewManager.recycle(mWebView);

    }
}
